#include <cstdio>
#include <vector>
#include <iostream>

const int INF = 1e9+1;

/* Popis nerovnosti.  */

class nerovnost
{
  int A, B, ell;
public:
  nerovnost (int _A, int _B, int _ell) : A (_A), B (_B), ell (_ell) {}

  /* Sniž hodnotu B tak, aby platila nerovnost HODNOTY[A] + ell >= HODNOTY[B].
     Vrací true, pokud hodnotu B bylo nutné měnit.  */
  bool vynut (std::vector<int> &hodnoty) const
    {
      if (hodnoty[A] + ell >= hodnoty[B])
        return false;

      hodnoty[B] = hodnoty[A] + ell;
      return true;
    }
};

/* Aktuální vzdálenost.  */
static std::vector<int> T;

/* Seznam nerovností.  */
static std::vector<nerovnost> nerovnosti;

/* Postupně vynutí platnost každé NEROVNOSTI (čímž ale může další nerovnosti
   pokazit), vrací true, pokud došlo k nějaké změně.  */
static bool
pruchod (const std::vector<nerovnost> &nerovnosti, std::vector<int> &hodnoty)
{
  bool ret = false;

  for (const nerovnost &n : nerovnosti)
    ret |= n.vynut (hodnoty);

  return ret;
}

/* Opakuje průchod, dokud se hodnoty mění, maximálně ale R-krát.
   Vrací true, pokud se v R-tém průchodu nic nezměnilo (nebo se
   průchodů provedlo méně).  */
static bool
opakuj_do_stabilizace (const std::vector<nerovnost> &nerovnosti,
                       std::vector<int> &hodnoty, int r)
{
  bool zmeneno = true;
  int iter;
  for (iter = 0; zmeneno && iter < r; iter++)
    {
      zmeneno = pruchod (nerovnosti, hodnoty);
    }

  /* Buď jsme skončili s iter < r, ZMENENO pak musí být false.
     Nebo probehlo iter = r průchodů a v posledním z nich se nic
     nezměnilo.  */
  return !zmeneno;
}

int main()
{
  int t, o;
  scanf ("%d%d", &t, &o);
  T.resize (t + 1, 0);
  nerovnosti.reserve (o + t);

  for (int i = 0; i < o; i++)
    {
      int A, B, m;
      scanf ("%d%d%d", &A, &B, &m);
      nerovnosti.emplace_back (A, B, -m);
    }

  /* Přidáme nerovnosti pro speciální vlak S = 0.  */
  for (int v = 1; v <= t; v++)
    nerovnosti.emplace_back (0, v, 0);

  if (opakuj_do_stabilizace (nerovnosti, T, t))
    {
      const char *sep = "";
      for (int v = 1; v <= t; v++)
        {
          printf ("%s%d", sep, T[v]);
          sep = " ";
        }
      printf ("\n");
    }
  else
    printf ("nelze\n");

  return 0;
}
